%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs Lornd University are Copyright 2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% @author Viktoria Fordos <v@fviktoria.hu>

%% -*- mode: nitrogen -*-

-module (nitrogen_lib).
-include_lib("referl_ui/include/nitrogen.hrl").
-include_lib("nitrogen/include/wf.hrl").
-include("records.hrl").
-vsn("$Rev: 9117 $ ").

-export([logout/1, logout_link/1, main/1]).

logout(User)->
    reflib_ui_router:del_msg_handler(wf:session(jobinfo)),
    wf:session(jobinfo,undefined),
    ?NITRO_SERVICES:delete_dependency_graphs_files(User),
    ?NITRO_SERVICES:stop_running_queries(User),
    wf:logout(),
    wf:redirect("/index").

main(TemplateName=[C|_]) when is_integer(C)->
    case reflib_ui_router:can_read_db() of
        true -> wf:wire(#toggle_access_to_db{enable=true});
        false -> 
            wf:wire(#toggle_access_to_db{enable=false}),
            wf:wire(#notification{type=notification,
                text="Database in use, most features are disabled."})
    end,
    Jobinfo=wf:session(jobinfo),
    if
        Jobinfo/=undefined ->
            reflib_ui_router:del_msg_handler(Jobinfo);
        true -> ok
    end,
    {ok,Pid}=wf:comet(fun() -> jobinfo_loop(TemplateName) end),
    reflib_ui_router:add_msg_handler(Pid),
    wf:session(jobinfo,Pid),
    case wf:role(auth_users) of
        true ->wf:session(flashes, undefined),
               #template { file=?NITRO_CORE:get_nitrogen_site_dir()
                         ++"/templates/"++TemplateName };
        false ->wf:redirect_to_login("/index")
    end.
    

jobinfo_loop(TemplateName) ->
    receive 
        {global,jobinfo,{{_,modifier,State,_},Msg}} when State==started ->
            wf:wire(#toggle_access_to_db{enable=false}),
            wf:wire(#notification{type=notification,text=Msg}),
            wf:flush(),
            jobinfo_loop(TemplateName);
        {global,jobinfo,{{_,modifier,State,_},Msg}} when State==finished ->
            ?NITRO_SERVICES:need_to_update(),
            if
                TemplateName=="main.html" -> main:update_source();
                true -> ok
            end,
            wf:wire(#toggle_access_to_db{enable=true}),
            wf:wire(#notification{type=notification,text=Msg}),
            wf:flush(),
            jobinfo_loop(TemplateName);
         _ -> jobinfo_loop(TemplateName)
    end.

logout_link(User)->
        #link {text="Log out "++User, 
           title = "Log out "++User,
           postback=logout }.
